/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <sys/resource.h>
#include <string.h>
#include <stdlib.h>

#include <oo.h>
#include <ooIndex.h>
#include "OO7.h"
#include "GenParams.h"
#include "VarParams.h"

extern int	debugMode;
extern char*	types[NumTypes];

static AtomicPart *old_atom = 0;

////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Constructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::AtomicPart(int32 ptId)
{
    if (debugMode) {
        printf("AtomicPart::AtomicPart(ptId = %d)\n", ptId);
    }

    // initialize internal state of new part (most of it randomly)
    id = ptId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAtomicDate +
    		(int) (random() % (MaxAtomicDate-MinAtomicDate+1));
    x = (int) (random() % XYRange);
    y = (int) (random() % XYRange);


    // fill in a random document id (for query 9)
    docId = (int) random() % TotalCompParts + 1;

    ooHandle(AtomicPart) atomH;
    (void) ooThis(atomH,oocRead);
    ooUpdateIndexes(atomH, 0, oocIOpInsert);
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart swapXY Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::swapXY()
{
    if (debugMode) {
        printf("                    AtomicPart::swapXY(ptId = %d, x = %d, y = %d)\n", id,x,y);
    }

    // exchange X and Y values

    ooUpdate();
    int tmp = x;
    x = y;
    y = tmp;

    if (debugMode) {
        printf("                    [did swap, so x = %d, y = %d]\n", x, y);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart toggleDate Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::toggleDate()
{
    if (debugMode) {
        printf("                    AtomicPart::toggleDate(ptId = %d, buildDate = %d)\n",
	       id, buildDate);
    }

    if (!old_atom) {
        old_atom = (AtomicPart *) malloc(sizeof(AtomicPart));
    }

    // increment build date if odd, decrement it if even
    ooHandle(AtomicPart) atomH;
    (void) ooThis(atomH);
    *old_atom = *(AtomicPart *) atomH;
    ooUpdate();


    if (buildDate % 2) {
	// odd case
	buildDate++;
    } else {
	// even case
	buildDate--;
    }

    ooUpdateIndexes(atomH, (char *) old_atom, oocIOpUpdate);

    if (debugMode) {
        printf("                    [did toggle, so buildDate = %d]\n", buildDate);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart DoNothing Method 
//
////////////////////////////////////////////////////////////////////////////

int RealWork; // set to one to make DoNothing do work.
int WorkAmount; // amount of work to do

void AtomicPart::DoNothing()
{
    struct timeval tv;
    struct timezone tz;
    int ignore;

    int i;

    if (id < 0) 
        printf("DoNothing: negative id.\n");

    if (debugMode) {
        printf("==> DoNothing(x = %d, y = %d, type = %s)\n", x, y, type);
    }

    if (RealWork) {
            for (i = 0; i < WorkAmount; i++) {
               gettimeofday(&tv, &tz);
            }
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Destructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::~AtomicPart()
{
    if (debugMode) {
        printf("AtomicPart::~AtomicPart(ptId = %d)\n", id);
    }

    // delete connections to other atomic parts (workaround due to
    // prop(delete) being broken at present for inline associations)

    ooItr(Connection) connI;

    for (to(connI); connI.next(); ) {
	if (!ooDeleteNoProp(connI)) {
	    fprintf(stderr, "ERROR: Unable to delete a to-connection\n");
	}
    }

    for (from(connI); connI.next(); ) {
	if (!ooDeleteNoProp(connI)) {
	    fprintf(stderr, "ERROR: Unable to delete a from-connection\n");
	}
    }
}
