/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Document Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include <oo.h>
#include "OO7.h"
#include "GenParams.h"
#include "VarParams.h"

extern int	debugMode;

////////////////////////////////////////////////////////////////////////////
//
// Document Constructor
//
////////////////////////////////////////////////////////////////////////////

Document::Document(int32 cpId)
{
    // fill in id

    id = cpId;

    // prepare and fill in the (indexed) document title

    sprintf(title, "Composite Part %08d", cpId);
    if (debugMode) {
        printf("Document::Document(title = %s)\n", title);
    }

    // prepare and fill in the document text

    char myText[DocTextLength];
    sprintf(myText, DocumentText, cpId);
    int stringLen = strlen(myText);
    text.update();
    text.resize(DocumentSize);
    int curChar, strChar;
    curChar = strChar = 0;
    char *memText = &text.elem(0);
    while (curChar < DocumentSize-1) {
       memText[curChar++] = myText[strChar++];
       if (strChar >= stringLen) { strChar = 0; }
    }
    memText[curChar] = '\0';
    ooHandle(Document) docH;
    (void) ooThis(docH,oocRead);
    ooUpdateIndexes(docH, 0, oocIOpInsert);
}


////////////////////////////////////////////////////////////////////////////
//
// Document searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::searchText(char c)
{
    if (debugMode) {
        printf("                    Document::searchText(title = %s)\n", title);
    }

    // count occurrences of the indicated letter (for busy work)

    int i, count = 0;
    int stringLen = text.size();
    char *memText = &text.elem(0);
    for (i = 0; i < stringLen; i++) {
	if (memText[i] == c) { count++; }
    }

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, i);
    }

    return count;
}


////////////////////////////////////////////////////////////////////////////
//
// Document replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::replaceText(char *oldString, char *newString)
{
    int i;

    if (debugMode) {
        printf("                    Document::changeText(title = %s)\n", title);
        printf("In replaceText, comparing \"");
        for (i = 0; i < 15; i++) { printf("%c", text[i]); }
        printf("\" with \"");
        for (i = 0; i < strlen(oldString); i++) { printf("%c", oldString[i]); }
        printf("\"\n");
    }

    // check to see if the text starts with the old string


    int oldTextLength = text.size();
    int oldStrLength  = strlen(oldString);
    int foundMatch = FALSE;
    if (oldStrLength < oldTextLength-1) {
        i = 0;
	while ((i < oldStrLength) && (text[i] == oldString[i])) { i++; }
	if (i == oldStrLength) { foundMatch = TRUE; };
    }

    // if so, change it to start with the new string instead

    if (foundMatch) {
        int newStrLength  = strlen(newString);
        int lengthDiff = newStrLength - oldStrLength;
        int newTextLength  = oldTextLength + lengthDiff;
        text.update();
        if (lengthDiff > 0) {
            text.resize(newTextLength);
            for (i = newTextLength - 1; i >= oldStrLength + lengthDiff; i--) {
                text[i] = text[i - lengthDiff];
            }
        } else {
            for (i = oldStrLength + lengthDiff; i < newTextLength; i++) {
                text[i] = text[i - lengthDiff];
            }
        }

        for (i = 0; i < newStrLength; i++) {
            text[i] = newString[i];
        }


       if (lengthDiff < 0) {
           text.resize(newTextLength); 
       }
    }

    if (debugMode) {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    if (foundMatch) {
        return 1;
    } else {
        return 0;
    }

}

