/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Module Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include <oo.h>
#include "OO7.h"
#include "GenParams.h"
#include "VarParams.h"

extern int	nextComplexAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

////////////////////////////////////////////////////////////////////////////
//
// Module Constructor
//
////////////////////////////////////////////////////////////////////////////

Module::Module(int32 modId)
{
    if (debugMode) {
        printf("Module::Module(modId = %d)\n", modId);
    }

    // initialize the simple stuff (some of it randomly)

    id = modId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinModuleDate +
    		(int) (random() % (MaxModuleDate - MinModuleDate + 1));

    // now create the assemblies for the module


    int assmId;
    ooHandle(Module) modH = ooThis();
    ooHandle(ComplexAssembly) nullPtr;
    nullPtr.nullify();

    if (NumAssmLevels > 1) {
        assmId = nextComplexAssemblyId++;
        ooHandle(ComplexAssembly) assmH = new(modH) ComplexAssembly
	    				  	    (assmId, nullPtr, 1);
        set_designRoot(assmH);
    }

    ooHandle(Manual) manH = new(modH) Manual(modId);
    set_man(manH);
}


///////////////////////////////////////////////////////////////////////////
// 
// Module::scanManual() returns the number of occurrences
// of the character 'I' in the module's manual.
//
///////////////////////////////////////////////////////////////////////////
int Module::scanManual() 
{
    return man()->searchText('I');
}

///////////////////////////////////////////////////////////////////////////
// 
// Module::firstLast() returns one if the first and last character
// of the manual are the same, zero otherwise.
//
///////////////////////////////////////////////////////////////////////////
int Module::firstLast() 
{
    return man()->firstLast();
}

