/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <oo.h>
#include <ooIndex.h>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #2 - 1% selection on AtomicParts via build date (the
// most recent 1% of AtomicParts.)
//
/////////////////////////////////////////////////////////////////


extern ooTrans	xact;
extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #2
//
//////////////////////////////////////////////////////////////////


int query2(ooHandle(ooDBObj)& databaseH)
{

    // get access to the design library containing atomic parts

    ooHandle(ooContObj) designLibH;
    designLibH.open(databaseH, "DesignLibrary", oocRead);
    if (designLibH.isValid() == oocFalse) {
        fprintf(stderr, "ERROR: Unable to access DesignLibrary\n");
	xact.abort();
        exit(1);
    }

    // choose a range of dates with the appropriate selectivity and cons
    // up a search predicate for the range

    int dateRange = (Query2Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    int lowerDate = MaxAtomicDate - dateRange;

    ooLookupKey datePred(ooTypeN(AtomicPart), 1);
    ooGreaterThanEqualLookupField dateLookupField(ooTypeN(AtomicPart), "buildDate", &lowerDate);
    datePred.addField(dateLookupField);

    if (debugMode) {
        printf("    Running Query2 with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    // iterate over all atomic parts in the design library (since this
    // release of Objectivity/DB does not include B+ trees) looking for
    // atomic parts that were built recently enough to be of interest

    int count = 0;
    ooItr(AtomicPart) partItr;

    for (partItr.scan(designLibH, datePred); partItr.next(); ) {

        if (debugMode) {
            printf("    In Query2, partId = %d, buildDate = %d\n",
	                partItr->id, partItr->buildDate);
        }

	// process qualifying parts by calling the null procedure
        partItr->DoNothing();
	count++;
    }
    return count;
}

