/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <oo.h>
#include <ooIndex.h>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #3 - 10% selection on AtomicParts via build date (the
// most recent 10% of AtomicParts.)   The actual percentage
// is determined by the constant Query3Percent.  This may
// be varied as we gain experience with the benchmark - the
// goal is to check out if the "optimizer" is smart enough
// to properly choose between B+ tree and scan. 
//
/////////////////////////////////////////////////////////////////


extern ooTrans	xact;
extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #3
//
//////////////////////////////////////////////////////////////////


int query3(ooHandle(ooDBObj)& databaseH)
{

    // get access to the design library containing atomic parts

    ooHandle(ooContObj) designLibH;
    designLibH.open(databaseH, "DesignLibrary", oocRead);
    if (designLibH.isValid() == oocFalse) {
        fprintf(stderr, "ERROR: Unable to access DesignLibrary\n");
	xact.abort();
        exit(1);
    }

    // choose a range of dates with the appropriate selectivity, and
    // cons up a predicate that specifies the desired range

    int dateRange = (Query3Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    int lowerDate = MaxAtomicDate - dateRange;
    ooLookupKey datePred(ooTypeN(AtomicPart), 1);
    ooGreaterThanEqualLookupField dateLookupField(ooTypeN(AtomicPart), "buildDate",&lowerDate);
    datePred.addField(dateLookupField);

    if (debugMode) {
        printf("    Running Query3 with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    // iterate over all atomic parts in the design library (since this
    // release of Objectivity/DB does not include B+ trees) looking for
    // atomic parts that were built recently enough to be of interest

    int count = 0;
    ooItr(AtomicPart) partItr;

    for (partItr.scan(designLibH, datePred); partItr.next(); ) {

        if (debugMode) {
            printf("    In Query1, partId = %d, buildDate = %d\n",
	                partItr->id, partItr->buildDate);
        }

	// process qualifying parts by calling the null procedure

	if (partItr->buildDate >= lowerDate) {
            partItr->DoNothing();
	    count++;
	}
    }
    return count;
}

