/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <oo.h>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #5 - find all base assemblies B that "use" a composite
// part with a more recent build date than B's build date.
//
/////////////////////////////////////////////////////////////////

extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #5
//
//////////////////////////////////////////////////////////////////


int query5(ooHandle(ooDBObj)& databaseH)
{
    // scan all modules, scanning all of the base assemblies in each one,
    // scanning all of the composite parts that they use, looking for base
    // assemblies that use more recently dated composite parts

    int count = 0;
    ooItr(Module) modI;
    for (modI.scan(databaseH); modI.next(); ) {

	ooItr(BaseAssembly) baseI;
	for (baseI.scan(modI); baseI.next(); ) {

            if (debugMode) {
                printf("    In Query5, processing base assembly %d, buildDate = %d\n",
		            baseI->id, baseI->buildDate);
            }

            ooItr(CompositePart) compI;
	    for (baseI->componentsPriv(compI); compI.next(); ) {

                if (debugMode) {
                    printf("            [Checking composite part %d, buildDate = %d]\n",
		                        compI->id, compI->buildDate);
                }

		if (compI->buildDate > baseI->buildDate) {
	            baseI->DoNothing();
		    count++;
	        }

	    }
	}
    }
    return count;
}

